const gulp = require('gulp')
const eslint = require('gulp-eslint')
const named = require('vinyl-named')
const path = require('path')
const plumber = require('gulp-plumber')
const stylus = require('gulp-stylus')
const webpack = require('webpack')
const webpackStream = require('webpack-stream')

const DEBUG = process.env.NODE_ENV !== 'production'

function end(done, debug = DEBUG) {
    return function () {
        if (!done.called && debug) {
            done.called = true
            done.call()
        }
    }
}

const options = {
    watch: DEBUG,
    output: {
        publicPath: '/'
    },
    resolve: {
        modules: ['node_modules'],
        extensions: ['.js']
    },
    module: {
        rules: [{
            test: /\.jsx?$/,
            exclude: /(node_modules)/,
            use: [{
                loader: 'babel-loader',
                options: {
                    babelrc: false,
                    cacheDirectory: true,
                    presets: ['es2015', 'stage-0', 'react'],
                    plugins: ['transform-runtime']
                }
            }]
        }, {
            test: /\.styl?$/,
            exclude: /(node_modules)/,
            use: [
                'style-loader',
                'css-loader',
                'stylus-loader'
            ]
        }]
    }
}

gulp.task('dev', ['dist:example', 'stylus', 'watch'], () => {
    require('gulp-nodemon')({
        script: './demo-server.js',
        ignore: [
            'dist/',
            'example/',
            'src/',
            'node_modules/'
        ]
    })
})

gulp.task('stylus', () => {
    return gulp.src('src/style.styl')
        .pipe(plumber())
        .pipe(stylus({ 'include css': true }))
        .pipe(gulp.dest('example'))
})

gulp.task('dist:default', done => {
    return gulp.src('src/index.js')
        .pipe(named())
        .pipe(webpackStream(options, webpack))
        .pipe(gulp.dest(path.join('dist')))
        .on('data', end(done, DEBUG))
})

gulp.task('dist:example', done => {
    return gulp.src('src/example.js')
        .pipe(named())
        .pipe(plumber())
        .pipe(webpackStream(options, webpack))
        .pipe(gulp.dest(path.join('example')))
        .on('data', end(done, DEBUG))
})

gulp.task('test', () => {
    return gulp.src('src/index.js')
        .pipe(eslint())
        .pipe(eslint.failAfterError())
})

gulp.task('watch', () => {
    gulp.watch('src/*.styl', ['stylus'])
})
