import React, { Component } from 'react'
import ReactDOM from 'react-dom'
import PropTypes from 'prop-types'

import './ripple.styl'

export default class Button extends Component {
    static defaultProps = {
        type: 'button',
        onClick: () => null,
        styles: {
            button: {},
            ripple: {},
            title: {}
        }
    }

    static propTypes = {
        type: PropTypes.string,
        onClick: PropTypes.func
    }

    handleClick = event => {
        const { isLoading, onClick } = this.props

        if (!isLoading && onClick) {
            onClick(event)
        }
    }

    handleMouseDown = event => {
        const { isLoading } = this.props

        const parent = ReactDOM.findDOMNode(this)
        const ripple = ReactDOM.findDOMNode(this.refs.ripple)
        const size = Math.max(parent.offsetWidth, parent.offsetHeight)

        ripple.classList.remove('ripple-button-react-button-animate')
        ripple.style.height = size + 'px'
        ripple.style.width = size + 'px'
        ripple.style.left = event.pageX - parent.offsetLeft - size/2 + 'px'
        ripple.style.top = event.pageY - parent.offsetTop - size/2 + 'px'
        ripple.classList.add('ripple-button-react-button-animate')
    }

    render() {
        const { isLoading, disabled, type, styles, ...props } = this.props

        return (
            <button className="ripple-button-react-button" onMouseDown={this.handleMouseDown}
                onClick={this.handleClick}
                type={type}
                style={styles.button}
                {...props}>
                <div className="ripple-button-react-button-ripple" style={styles.ripple} ref="ripple"></div>
                <div className="ripple-button-react-button-title" style={styles.title}>{this.props.children}</div>
            </button>
        )
    }
}
